@@ -8,6 +8,7 @@
  *	Based on ff-csv code.
  */
 #include <config.h>
+#include <sys/types.h>
 #include <sys/mman.h>
 #include <unistd.h>
 #include <stdio.h>
@@ -325,7 +326,7 @@ dif_file_open (GnumFileOpener const *fo, IOContext *io_context,
 	if (ctxt != NULL) {
 		dif_parse_sheet (ctxt);
 		if (gnumeric_io_error_occurred (io_context)) {
-			gnumeric_io_error_info_push (io_context, error_info_new_str (
+			gnumeric_io_error_push (io_context, error_info_new_str (
 			_("Error while reading DIF file.")));
 		}
 		dif_input_context_destroy (ctxt);
@@ -369,7 +370,7 @@ dif_file_save (GnumFileSaver const *fs, IOContext *io_context,
 	fputs ("DATA\n0,0\n" "\"\"\n", f);
 
 	/* Process all cells */
-	r = sheet_get_extent (sheet);
+	r = sheet_get_extent (sheet, FALSE);
 	for (row = r.start.row; row <= r.end.row; row++) {
 		fputs ("-1,0\n" "BOT\n", f);
 		for (col = r.start.col; col <= r.end.col; col++) {
