@@ -25,8 +25,7 @@
 #include "gnumeric.h"
 #include "gnumeric-util.h"
 
-int html_write_wb_latex   (IOContext *context, WorkbookView *wb_view,
-                           const char *filename, gpointer user_data);
-int html_write_wb_latex2e (IOContext *context, WorkbookView *wb_view,
-                           const char *filename, gpointer user_data);
+void latex_file_save (GnumFileSaver const *fs, IOContext *io_context, WorkbookView *wb_view, const gchar *file_name);
+void latex2e_file_save (GnumFileSaver const *fs, IOContext *io_context, WorkbookView *wb_view, const gchar *file_name);
+
 #endif
