@@ -1,3 +1,4 @@
+/* vim: set sw=8: */
 /*
  * selection.c:  Manage selection regions.
  *
@@ -5,10 +6,11 @@
  *  Miguel de Icaza (miguel@gnu.org)
  *  Jody Goldberg (jgoldberg@home.com)
  *
- *  (C) 1999, 2000 Jody Goldberg
+ *  (C) 1999-2001 Jody Goldberg
  */
 #include <config.h>
 #include "selection.h"
+#include "sheet.h"
 #include "parse-util.h"
 #include "clipboard.h"
 #include "ranges.h"
@@ -22,7 +24,7 @@
 #include "cell.h"
 #include "sheet-merge.h"
 #include "sheet-private.h"
-#include "sheet-control-gui.h"
+#include "sheet-control.h"
 #include "gnumeric-util.h"
 
 /*
@@ -55,84 +57,34 @@ segments_intersect (int const s_a, int const e_a,
 }
 
 /**
- * Return 1st range.
- * Return NULL if there is more than 1 and @permit_complex is FALSE
- **/
-Range const *
-selection_first_range (Sheet const *sheet, gboolean const permit_complex)
-{
-	Range *ss;
-	GList *l;
-
-	g_return_val_if_fail (IS_SHEET (sheet), 0);
-
-	l = g_list_first (sheet->selections);
-	if (!l || !l->data)
-		return NULL;
-	ss = l->data;
-	if ((l = g_list_next (l)) && !permit_complex)
-		return NULL;
-
-	return ss;
-}
-
-static Value *
-cb_cell_is_array (Sheet *sheet, int col, int row, Cell *cell, void *user_data)
-{
-	return cell_is_array (cell) ? value_terminate () : NULL;
-
-}
-
-/*
- * selection_is_simple
- * @wbc          : The calling context to report errors to (GUI or corba)
+ * selection_first_range
  * @sheet        : The sheet whose selection we are testing.
+ * @wbc          : The calling context to report errors to (GUI or corba)
  * @command_name : A string naming the operation requiring a single range.
  *
- * This function tests to see if multiple ranges are selected.  If so it
- * produces a warning.  It also ensures that the range does not contain any
- * arrays or merged cells.
+ * Returns the first range, if a control is supplied it displays an error if
+ *    there is more than one range.
  */
-gboolean
-selection_is_simple (WorkbookControl *wbc, Sheet const *sheet,
-		     char const *command_name,
-		     gboolean allow_merged, gboolean allow_arrays)
+Range const *
+selection_first_range (Sheet const *sheet, 
+		       WorkbookControl *wbc, char const *cmd_name)
 {
 	Range const *r;
-	GSList *merged;
+	GList *l;
 
-	g_return_val_if_fail (IS_SHEET (sheet), FALSE);
+	g_return_val_if_fail (IS_SHEET (sheet), NULL);
 
-	if (g_list_length (sheet->selections) != 1) {
-		gnumeric_error_invalid (COMMAND_CONTEXT (wbc), command_name,
-			_("cannot be performed with multiple selections"));
-		return FALSE;
+	l = g_list_first (sheet->selections);
+	g_return_val_if_fail (l != NULL && l->data != NULL, NULL);
+
+	r = l->data;
+	if (wbc != NULL && l->next != NULL) {
+		gnumeric_error_invalid (COMMAND_CONTEXT (wbc), cmd_name,
+			_("cannot be performed with multiple ranges selected"));
+		return NULL;
 	}
 
-	r = sheet->selections->data;
-
-	if (!allow_merged) {
-		merged = sheet_merge_get_overlap (sheet, r);
-		if (merged != NULL) {
-			gnumeric_error_invalid (COMMAND_CONTEXT (wbc), command_name,
-				_("can not operate on merged cells"));
-			g_slist_free (merged);
-			return FALSE;
-		}
-	}
-
-	if (!allow_arrays) {
-		if (sheet_foreach_cell_in_range ((Sheet *)sheet, TRUE,
-					      r->start.col, r->start.row,
-					      r->end.col, r->end.row,
-					      cb_cell_is_array, NULL)) {
-			gnumeric_error_invalid (COMMAND_CONTEXT (wbc), command_name,
-				_("can not operate on array formulae"));
-			return FALSE;
-		}
-	}
-
-	return TRUE;
+	return r;
 }
 
 /**
@@ -186,40 +138,8 @@ sheet_selection_extend_to (Sheet *sheet, int col, int row)
 	});
 }
 
-/*
- * sheet_selection_extend
- * @sheet              : Sheet to operate in.
- * @n                  : Units to extend the selection
- * @jump_to_boundaries : Move to transitions between cells and blanks,
- *                       or move in single steps.
- * @horizontal         : extend vertically or horizontally.
- */
-void
-sheet_selection_extend (Sheet *sheet, int n, gboolean jump_to_boundaries,
-			gboolean const horizontal)
-{
-	/* Use a tmp variable so that the extend_to routine knows that the
-	 * selection has actually changed.
-	 */
-	CellPos tmp = sheet->cursor.move_corner;
-
-	if (horizontal)
-		tmp.col = sheet_find_boundary_horizontal (sheet,
-				tmp.col, tmp.row,
-				sheet->cursor.base_corner.row,
-				n, jump_to_boundaries);
-	else
-		tmp.row = sheet_find_boundary_vertical (sheet,
-				tmp.col, tmp.row,
-				sheet->cursor.base_corner.col,
-				n, jump_to_boundaries);
-
-	sheet_selection_extend_to (sheet, tmp.col, tmp.row);
-	sheet_make_cell_visible (sheet, tmp.col, tmp.row);
-}
-
 gboolean
-sheet_is_all_selected (Sheet const * const sheet)
+sheet_is_all_selected (Sheet const *sheet)
 {
 	Range *ss;
 	GList *l;
@@ -239,7 +159,7 @@ sheet_is_all_selected (Sheet const * const sheet)
 }
 
 gboolean
-sheet_is_cell_selected (Sheet const * const sheet, int col, int row)
+sheet_is_cell_selected (Sheet const *sheet, int col, int row)
 {
 	GList *list;
 
@@ -261,10 +181,10 @@ sheet_selection_redraw (Sheet const *sheet)
 		Range const *r = sel->data;
 
 		SHEET_FOREACH_CONTROL (sheet, control,
-			scg_redraw_cell_region (control,
+			sc_redraw_region (control,
 				r->start.col, r->start.row,
 				r->end.col, r->end.row);
-			scg_redraw_headers (control, TRUE, TRUE, r););
+			sc_redraw_headers (control, TRUE, TRUE, r););
 	}
 }
 
@@ -272,7 +192,7 @@ sheet_selection_redraw (Sheet const *sheet)
  * TRUE : If the range overlaps with any part of the selection
  */
 gboolean
-sheet_is_range_selected (Sheet const * const sheet, Range const *r)
+sheet_is_range_selected (Sheet const *sheet, Range const *r)
 {
 	GList *list;
 
@@ -289,7 +209,7 @@ sheet_is_range_selected (Sheet const * const sheet, Range const *r)
  * TRUE : If entire range is contained in the selection
  */
 gboolean
-sheet_is_full_range_selected (Sheet const * const sheet, Range const *r)
+sheet_is_full_range_selected (Sheet const *sheet, Range const *r)
 {
 	GList *list;
 
@@ -309,9 +229,7 @@ sheet_selection_set_internal (Sheet *sheet,
 			      int move_col, int move_row,
 			      gboolean just_add_it)
 {
-	GSList *merged, *ptr;
 	GList *list;
-	gboolean changed;
 	Range *ss;
 	Range old_sel, new_sel;
 	gboolean do_cols, do_rows;
@@ -319,39 +237,15 @@ sheet_selection_set_internal (Sheet *sheet,
 	g_return_if_fail (IS_SHEET (sheet));
 	g_return_if_fail (sheet->selections != NULL);
 
-	ss = (Range *)sheet->selections->data;
-
 	new_sel.start.col = MIN(base_col, move_col);
 	new_sel.start.row = MIN(base_row, move_row);
 	new_sel.end.col = MAX(base_col, move_col);
 	new_sel.end.row = MAX(base_row, move_row);
 
-	/* expand to include any merged regions */
-	do {
-		changed = FALSE;
-		merged = sheet_merge_get_overlap (sheet, &new_sel);
-		for (ptr = merged ; ptr != NULL ; ptr = ptr->next) {
-			Range const *r = ptr->data;
-			if (new_sel.start.col > r->start.col) {
-				new_sel.start.col = r->start.col;
-				changed = TRUE;
-			}
-			if (new_sel.start.row > r->start.row) {
-				new_sel.start.row = r->start.row;
-				changed = TRUE;
-			}
-			if (new_sel.end.col < r->end.col) {
-				new_sel.end.col = r->end.col;
-				changed = TRUE;
-			}
-			if (new_sel.end.row < r->end.row) {
-				new_sel.end.row = r->end.row;
-				changed = TRUE;
-			}
-		}
-		g_slist_free (merged);
-	} while (changed);
+	g_return_if_fail (range_is_sane (&new_sel));
 
+	sheet_merge_find_container (sheet, &new_sel);
+	ss = (Range *)sheet->selections->data;
 	if (!just_add_it && range_equal (ss, &new_sel))
 		return;
 
@@ -359,10 +253,10 @@ sheet_selection_set_internal (Sheet *sheet,
 	*ss = new_sel;
 
 	/* Set the cursor boundary */
-	sheet_cursor_set_full (sheet,
-			       edit_col, edit_row,
-			       base_col, base_row,
-			       move_col, move_row, ss);
+	sheet_cursor_set (sheet,
+			  edit_col, edit_row,
+			  base_col, base_row,
+			  move_col, move_row, ss);
 
 	if (just_add_it) {
 		sheet_redraw_range (sheet, &new_sel);
@@ -465,16 +359,31 @@ set_menu_flags:
 	 * insert row/cols and the flags accordingly.
 	 */
 	do_rows = do_cols = TRUE;
-	for (list = sheet->selections; list && (do_cols || do_rows); list = list->next){
-		Range *r = list->data;
+	for (list = sheet->selections; list && (do_cols || do_rows); list = list->next) {
+		Range const *r = list->data;
 
-		if (do_rows && r->start.row == 0 && r->end.row == SHEET_MAX_ROWS - 1)
-			do_rows = FALSE;
-
-		if (do_cols && r->start.col == 0 && r->end.col == SHEET_MAX_COLS - 1)
+		if (do_cols && range_is_full (r, TRUE))
 			do_cols = FALSE;
+		if (do_rows && range_is_full (r, FALSE))
+			do_rows = FALSE;
 	}
 	sheet_menu_state_enable_insert (sheet, do_cols, do_rows);
+
+	/*
+	 * FIXME: Enable/disable the show/hide detail menu items here.
+	 * We can only do this when the data structures have improved, currently
+	 * checking for this will be to slow.
+	 * Once it works, use this code :
+	 *
+	 * sheet->priv->enable_showhide_detail = ....
+	 *
+	 * WORKBOOK_FOREACH_VIEW (sheet->workbook, view, {
+	 *	if (sheet == wb_view_cur_sheet (view)) {
+	 *	        WORKBOOK_VIEW_FOREACH_CONTROL(view, wbc,
+	 *	                wb_control_menu_state_update (wbc, sheet, MS_SHOWHIDE_DETAIL););
+ 	 *      }
+	 * });
+	 */
 }
 
 void
@@ -581,14 +490,10 @@ sheet_selection_reset (Sheet *sheet)
 static void
 reference_append (GString *result_str, CellPos const *pos)
 {
-	char *row_string = g_strdup_printf ("%d", pos->row);
-
 	g_string_append_c (result_str, '$');
 	g_string_append (result_str, col_name (pos->col));
 	g_string_append_c (result_str, '$');
-	g_string_append (result_str, row_string);
-
-	g_free (row_string);
+	g_string_append (result_str, row_name (pos->row));
 }
 
 char *
@@ -626,40 +531,15 @@ sheet_selection_to_string (Sheet *sheet, gboolean include_sheet_name_prefix)
 	return result;
 }
 
-void
-sheet_selection_ant (Sheet *sheet)
-{
-	g_return_if_fail (IS_SHEET (sheet));
-
-	SHEET_FOREACH_CONTROL (sheet, control,
-		scg_selection_ant (control););
-}
-
-void
-sheet_selection_unant (Sheet *sheet)
-{
-	g_return_if_fail (IS_SHEET (sheet));
-
-	SHEET_FOREACH_CONTROL (sheet, control,
-		scg_selection_unant (control););
-}
-
 gboolean
 sheet_selection_copy (WorkbookControl *wbc, Sheet *sheet)
 {
-	Range *ss;
+	Range const *sel;
 
-	g_return_val_if_fail (IS_SHEET (sheet), FALSE);
-	g_return_val_if_fail (sheet->selections, FALSE);
-
-	/* FIXME FIXME : disable copying part of an array */
-
-	if (!selection_is_simple (wbc, sheet, _("Copy"), TRUE, TRUE))
+	if (!(sel = selection_first_range (sheet, wbc, _("Copy"))))
 		return FALSE;
 
-	ss = sheet->selections->data;
-
-	application_clipboard_copy (wbc, sheet, ss);
+	application_clipboard_copy (wbc, sheet, sel);
 
 	return TRUE;
 }
@@ -667,7 +547,7 @@ sheet_selection_copy (WorkbookControl *wbc, Sheet *sheet)
 gboolean
 sheet_selection_cut (WorkbookControl *wbc, Sheet *sheet)
 {
-	Range *ss;
+	Range const *sel;
 
 	/*
 	 * 'cut' is a poor description of what we're
@@ -682,14 +562,13 @@ sheet_selection_cut (WorkbookControl *wbc, Sheet *sheet)
 	 */
 	g_return_val_if_fail (IS_SHEET (sheet), FALSE);
 
-	if (!selection_is_simple (wbc, sheet, _("Cut"), TRUE, TRUE))
+	if (!(sel = selection_first_range (sheet, wbc, _("Cut"))))
 		return FALSE;
 
-	ss = sheet->selections->data;
-	if (sheet_range_splits_array (sheet, ss, wbc, ("cut")))
+	if (sheet_range_splits_region (sheet, sel, NULL, wbc, _("Cut")))
 		return FALSE;
 
-	application_clipboard_cut (wbc, sheet, ss);
+	application_clipboard_cut (wbc, sheet, sel);
 
 	return TRUE;
 }
@@ -700,7 +579,7 @@ sheet_selection_cut (WorkbookControl *wbc, Sheet *sheet)
  * @allow_intersection : Divide the selection into nonoverlapping subranges.
  */
 GSList *
-selection_get_ranges (Sheet const *sheet, gboolean const allow_intersection)
+selection_get_ranges (Sheet const *sheet, gboolean allow_intersection)
 {
 	GList  *l;
 	GSList *proposed = NULL;
@@ -716,12 +595,12 @@ selection_get_ranges (Sheet const *sheet, gboolean const allow_intersection)
 	 * single user proposed segment and accumulate distict regions.
 	 */
 	for (l = sheet->selections; l != NULL; l = l->next) {
-		Range const *ss = l->data;
+		Range const *r = l->data;
 
 		/* The set of regions that do not interset with b or
 		 * its predecessors */
 		GSList *clear = NULL;
-		Range *tmp, *b = range_copy (ss);
+		Range *tmp, *b = range_dup (r);
 
 		if (allow_intersection) {
 			proposed = g_slist_prepend (proposed, b);
@@ -776,12 +655,12 @@ selection_get_ranges (Sheet const *sheet, gboolean const allow_intersection)
 				segments_intersect (a->start.row, a->end.row,
 						    b->start.row, b->end.row);
 #ifdef DEBUG_SELECTION
-			fprintf (stderr, "row = %d\na = %d", row_intersect, a->start.row +1);
+			fprintf (stderr, "row = %d\na = %s", row_intersect, row_name (a->start.row));
 			if (a->start.row != a->end.row)
-				fprintf (stderr, " -> %d", a->end.row +1);
-			fprintf (stderr, "\nb = %d", b->start.row +1);
+				fprintf (stderr, " -> %s", row_name (a->end.row));
+			fprintf (stderr, "\nb = %s", row_name (b->start.row));
 			if (b->start.row != b->end.row)
-				fprintf (stderr, " -> %d\n", b->end.row +1);
+				fprintf (stderr, " -> %s\n", row_name (b->end.row));
 			else
 				fputc ('\n', stderr);
 #endif
@@ -832,7 +711,7 @@ selection_get_ranges (Sheet const *sheet, gboolean const allow_intersection)
 					}
 					if (a->start.col != b->start.col) {
 						/* Split existing range */
-						tmp = range_copy (a);
+						tmp = range_dup (a);
 						tmp->end.col = b->start.col - 1;
 						clear = g_slist_prepend (clear, tmp);
 					}
@@ -860,7 +739,7 @@ selection_get_ranges (Sheet const *sheet, gboolean const allow_intersection)
 				case 3 : /* overlap top */
 					/* Split region */
 					if (b->start.row > 0) {
-						tmp = range_copy (a);
+						tmp = range_dup (a);
 						tmp->start.col = b->start.col;
 						tmp->end.row = b->start.row - 1;
 						clear = g_slist_prepend (clear, tmp);
@@ -875,7 +754,7 @@ selection_get_ranges (Sheet const *sheet, gboolean const allow_intersection)
 				case 1 : /* overlap bottom */
 					/* Split region */
 					if (b->end.row < (SHEET_MAX_ROWS-1)) {
-						tmp = range_copy (a);
+						tmp = range_dup (a);
 						tmp->start.col = b->start.col;
 						tmp->start.row = b->end.row + 1;
 						clear = g_slist_prepend (clear, tmp);
@@ -916,7 +795,7 @@ selection_get_ranges (Sheet const *sheet, gboolean const allow_intersection)
 					}
 					if (a->start.row != b->start.row) {
 						/* Split region */
-						tmp = range_copy (a);
+						tmp = range_dup (a);
 						tmp->end.row = b->start.row - 1;
 						clear = g_slist_prepend (clear, tmp);
 					}
@@ -941,7 +820,7 @@ selection_get_ranges (Sheet const *sheet, gboolean const allow_intersection)
 
 				case 3 : /* overlap top */
 					/* Split region */
-					tmp = range_copy (a);
+					tmp = range_dup (a);
 					tmp->end.col = b->end.col;
 					tmp->end.row = b->start.row - 1;
 					clear = g_slist_prepend (clear, tmp);
@@ -954,7 +833,7 @@ selection_get_ranges (Sheet const *sheet, gboolean const allow_intersection)
 
 				case 1 : /* overlap bottom */
 					/* Split region */
-					tmp = range_copy (a);
+					tmp = range_dup (a);
 					tmp->end.col = b->end.col;
 					tmp->start.row = b->end.row + 1;
 
@@ -981,30 +860,6 @@ selection_get_ranges (Sheet const *sheet, gboolean const allow_intersection)
 	return proposed;
 }
 
-/**
- * selection_check_for_array:
- * @sheet: the sheet.
- * @selection : A list of ranges to check.
- * @wbc : The context that issued the command
- * @cmd : The translated command name.
- *
- * A utility to check whether the selection contains any partial arrays.
- */
-gboolean
-selection_check_for_array (Sheet const * sheet, GSList const *selection,
-			   WorkbookControl *wbc, char const *cmd)
-{
-	GSList const *l;
-
-	/* Check for array subdivision */
-	for (l = selection; l != NULL; l = l->next) {
-		Range const *r = l->data;
-		if (sheet_range_splits_array (sheet, r, wbc, cmd))
-			return TRUE;
-	}
-	return FALSE;
-}
-
 /**
  * selection_apply:
  * @sheet: the sheet.
@@ -1066,11 +921,11 @@ cb_range_to_string (Sheet *sheet, Range const *r, void *closure)
 	if (res->include_sheet_name_prefix)
 		g_string_sprintfa (res->str, "%s!", sheet->name_quoted);
 
-	g_string_sprintfa (res->str, "$%s$%d",
-			   col_name (r->start.col), r->start.row+1);
+	g_string_sprintfa (res->str, "$%s$%s",
+			   col_name (r->start.col), row_name (r->start.row));
 	if ((r->start.col != r->end.col) || (r->start.row != r->end.row))
-		g_string_sprintfa (res->str, ":$%s$%d",
-				   col_name (r->end.col), r->end.row+1);
+		g_string_sprintfa (res->str, ":$%s$%s",
+				   col_name (r->end.col), row_name (r->end.row));
 }
 
 char *
@@ -1251,15 +1106,15 @@ loop :
 
 void
 sheet_selection_walk_step (Sheet *sheet,
-			   gboolean const forward,
-			   gboolean const horizontal)
+			   gboolean forward,
+			   gboolean horizontal)
 {
 	int selections_count;
 	CellPos destination;
 	Range const *ss;
 	gboolean is_singleton = FALSE;
 
-	g_return_if_fail (sheet != NULL);
+	g_return_if_fail (IS_SHEET (sheet));
 	g_return_if_fail (sheet->selections != NULL);
 
 	ss = sheet->selections->data;
@@ -1333,7 +1188,7 @@ sheet_selection_walk_step (Sheet *sheet,
 			sheet_cursor_set (sheet,
 					  destination.col, destination.row,
 					  ss->start.col, ss->start.row,
-					  ss->end.col, ss->end.row);
+					  ss->end.col, ss->end.row, NULL);
 	}
 
 	sheet_set_edit_pos (sheet, destination.col, destination.row);
