@@ -12,7 +12,6 @@
 #include <gnome.h>
 #include "gnumeric.h"
 #include "sheet-control-gui.h"
-#include "gnumeric-type-util.h"
 #include "dialogs.h"
 #include "sheet-object-impl.h"
 #include "workbook-edit.h"
@@ -28,6 +27,8 @@
 #include "sheet-object-bonobo.h"
 #endif
 
+#include <gal/util/e-util.h>
+
 /* Returns the class for a SheetObject */
 #define SO_CLASS(so) SHEET_OBJECT_CLASS(GTK_OBJECT(so)->klass)
 
@@ -51,8 +52,8 @@ cb_sheet_object_configure (GtkWidget *widget, GtkObject *obj_view)
 
 	g_return_if_fail (obj_view != NULL);
 
-	so = gtk_object_get_data (obj_view, SO_VIEW_OBJECT_KEY);
-	scg = gtk_object_get_data (obj_view, SO_VIEW_SHEET_CONTROL_KEY);
+	so = sheet_object_view_obj (obj_view);
+	scg = sheet_object_view_control (obj_view);
 
 	SO_CLASS(so)->user_config (so, scg);
 }
@@ -194,9 +195,9 @@ sheet_object_class_init (GtkObjectClass *object_class)
 	sheet_object_class->default_height_pts = 36.;	/* 1/2 inch */
 }
 
-GNUMERIC_MAKE_TYPE (sheet_object, "SheetObject", SheetObject,
-		    sheet_object_class_init, sheet_object_init,
-		    gtk_object_get_type ())
+E_MAKE_TYPE (sheet_object, "SheetObject", SheetObject,
+	     sheet_object_class_init, sheet_object_init,
+	     GTK_TYPE_OBJECT);
 
 SheetObject *
 sheet_object_view_obj (GtkObject *view)
@@ -304,7 +305,11 @@ sheet_object_clear_sheet (SheetObject *so)
 static void
 sheet_object_view_destroyed (GtkObject *view, SheetObject *so)
 {
+	SheetControlGUI *scg = sheet_object_view_control (view);
+
 	so->realized_list = g_list_remove (so->realized_list, view);
+	gtk_object_remove_data	(view, SO_VIEW_SHEET_CONTROL_KEY);
+	gtk_object_unref (GTK_OBJECT (scg));
 }
 
 /*
@@ -328,6 +333,7 @@ sheet_object_new_view (SheetObject *so, SheetControlGUI *scg)
 	/* Store some useful information */
 	gtk_object_set_data (GTK_OBJECT (view), SO_VIEW_OBJECT_KEY, so);
 	gtk_object_set_data (GTK_OBJECT (view), SO_VIEW_SHEET_CONTROL_KEY, scg);
+	gtk_object_ref (GTK_OBJECT (scg));
 
 	gtk_signal_connect (GTK_OBJECT (view), "destroy",
 			    GTK_SIGNAL_FUNC (sheet_object_view_destroyed), so);
@@ -499,9 +505,7 @@ static int
 cell_offset_calc_pixel (Sheet const *sheet, int i, gboolean is_col,
 			SheetObjectAnchor anchor_type, float offset)
 {
-	ColRowInfo const *cri = is_col
-		? sheet_col_get_info (sheet, i)
-		: sheet_row_get_info (sheet, i);
+	ColRowInfo const *cri = sheet_colrow_get_info (sheet, i, is_col);
 	/* TODO : handle other anchor types */
 	if (anchor_type == SO_ANCHOR_PERCENTAGE_FROM_COLROW_END)
 		return (1. - offset) * cri->size_pixels;
@@ -522,7 +526,7 @@ sheet_object_position_pixels (SheetObject const *so,
 			      SheetControlGUI const *scg, int *coords)
 {
 	g_return_if_fail (IS_SHEET_OBJECT (so));
-	g_return_if_fail (so->sheet != NULL);
+	g_return_if_fail (IS_SHEET (so->sheet));
 
 	coords [0] = scg_colrow_distance_get (scg, TRUE, 0,
 		so->cell_bound.start.col);
@@ -548,9 +552,7 @@ static double
 cell_offset_calc_pt (Sheet const *sheet, int i, gboolean is_col,
 		     SheetObjectAnchor anchor_type, float offset)
 {
-	ColRowInfo const *cri = is_col
-		? sheet_col_get_info (sheet, i)
-		: sheet_row_get_info (sheet, i);
+	ColRowInfo const *cri = sheet_colrow_get_info (sheet, i, is_col);
 	/* TODO : handle other anchor types */
 	if (anchor_type == SO_ANCHOR_PERCENTAGE_FROM_COLROW_END)
 		return (1. - offset) * cri->size_pixels;
