@@ -1,3 +1,4 @@
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * value.c:  Utilies for handling, creating, removing values.
  *
@@ -8,15 +9,17 @@
  */
 
 #include <config.h>
-#include <gnome.h>
-#include "gnumeric.h"
 #include "value.h"
 #include "parse-util.h"
 #include "style.h"
 #include "format.h"
 #include "portability.h"
+#include "str.h"
+
 #include <stdlib.h>
 #include <errno.h>
+#include <math.h>
+#include <gnome.h>
 
 Value *
 value_new_empty (void)
@@ -71,8 +74,15 @@ value_new_error_str (EvalPos const *ep, String *mesg)
 	return (Value *)v;
 }
 
+/**
+ * value_error_set_pos :
+ * @err :
+ * @pos :
+ *
+ * Change the position of a ValueError.
+ */
 Value *
-value_new_error_err (EvalPos const *pos, ValueErr *err)
+value_error_set_pos (ValueErr *err, EvalPos const *pos)
 {
     g_return_val_if_fail (err != NULL, NULL);
     g_return_val_if_fail (err->type == VALUE_ERROR, NULL);
@@ -82,7 +92,7 @@ value_new_error_err (EvalPos const *pos, ValueErr *err)
 }
 
 Value *
-value_new_string (const char *str)
+value_new_string (char const *str)
 {
 	ValueStr *v = g_new (ValueStr, 1);
 	*((ValueType *)&(v->type)) = VALUE_STRING;
@@ -100,7 +110,7 @@ value_new_string_str (String *str)
 }
 
 Value *
-value_new_cellrange_unsafe (const CellRef *a, const CellRef *b)
+value_new_cellrange_unsafe (CellRef const *a, CellRef const *b)
 {
 	ValueRange *v = g_new (ValueRange, 1);
 	*((ValueType *)&(v->type)) = VALUE_CELLRANGE;
@@ -119,8 +129,8 @@ value_new_cellrange_unsafe (const CellRef *a, const CellRef *b)
  * evaluate the ranges in their context and normalize then.
  */
 Value *
-value_new_cellrange (const CellRef *a, const CellRef *b,
-		     int const eval_col, int const eval_row)
+value_new_cellrange (CellRef const *a, CellRef const *b,
+		     int eval_col, int eval_row)
 {
 	ValueRange *v = g_new (ValueRange, 1);
 	int tmp;
@@ -164,7 +174,7 @@ value_new_cellrange (const CellRef *a, const CellRef *b,
 }
 
 Value *
-value_new_cellrange_r (Sheet *sheet, const Range *r)
+value_new_cellrange_r (Sheet *sheet, Range const *r)
 {
 	ValueRange *v = g_new (ValueRange, 1);
 	CellRef *a, *b;
@@ -199,7 +209,7 @@ value_new_array_non_init (guint cols, guint rows)
 Value *
 value_new_array (guint cols, guint rows)
 {
-	int x, y;
+	guint x, y;
 	ValueArray *v = (ValueArray *)value_new_array_non_init (cols, rows);
 
 	for (x = 0; x < cols; x++) {
@@ -213,7 +223,7 @@ value_new_array (guint cols, guint rows)
 Value *
 value_new_array_empty (guint cols, guint rows)
 {
-	int x, y;
+	guint x, y;
 	ValueArray *v = (ValueArray *)value_new_array_non_init (cols, rows);
 
 	for (x = 0; x < cols; x++) {
@@ -225,7 +235,7 @@ value_new_array_empty (guint cols, guint rows)
 }
 
 Value *
-value_new_from_string (ValueType t, const char *str)
+value_new_from_string (ValueType t, char const *str)
 {
 	switch (t) {
 	case VALUE_EMPTY:
@@ -315,7 +325,7 @@ value_release (Value *value)
 
 	case VALUE_ARRAY: {
 		ValueArray *v = (ValueArray *)value;
-		guint x, y;
+		int x, y;
 
 		for (x = 0; x < v->x; x++) {
 			for (y = 0; y < v->y; y++) {
@@ -350,7 +360,7 @@ value_release (Value *value)
  * Makes a copy of a Value
  */
 Value *
-value_duplicate (const Value *src)
+value_duplicate (Value const *src)
 {
 	g_return_val_if_fail (src != NULL, NULL);
 
@@ -453,7 +463,7 @@ value_get_as_checked_bool (Value const *v)
  * simplistic value rendering
  */
 char *
-value_get_as_string (const Value *value)
+value_get_as_string (Value const *value)
 {
 	if (value == NULL)
 		return g_strdup ("");
@@ -481,12 +491,12 @@ value_get_as_string (const Value *value)
 		char const row_sep = format_get_arg_sep ();
 		char const col_sep = format_get_col_sep ();
 		GString *str = g_string_new ("{");
-		guint x, y;
+		int x, y;
 		char *ans;
 
 		for (y = 0; y < value->v_array.y; y++){
 			for (x = 0; x < value->v_array.x; x++){
-				const Value *v = value->v_array.vals [x][y];
+				Value const *v = value->v_array.vals [x][y];
 
 				g_return_val_if_fail (v->type == VALUE_STRING ||
 						      v->type == VALUE_FLOAT ||
@@ -521,11 +531,36 @@ value_get_as_string (const Value *value)
 	return g_strdup ("Internal problem");
 }
 
+/*
+ * Result will stay valid until (a) the value is disposed of, or (b) two
+ * further calls to this function are made.
+ */
+char const *
+value_peek_string (Value const *v)
+{
+	g_return_val_if_fail (v, "");
+
+	if (v->type == VALUE_STRING)
+		return v->v_str.val->str;
+	else {
+		static char *cache[2] = { 0 };
+		static int next = 0;
+		char const *s;
+
+		g_free (cache[next]);
+		s = cache[next] = value_get_as_string (v);
+		next = (next + 1) % (sizeof (cache) / sizeof (cache[0]));
+		return s;
+	}
+}
+
+
+
 /*
  * FIXME FIXME FIXME : Support errors
  */
 int
-value_get_as_int (const Value *v)
+value_get_as_int (Value const *v)
 {
 	if (v == NULL)
 		return 0;
@@ -566,7 +601,7 @@ value_get_as_int (const Value *v)
  * FIXME FIXME FIXME : Support errors
  */
 gnum_float
-value_get_as_float (const Value *v)
+value_get_as_float (Value const *v)
 {
 	if (v == NULL)
 		return 0.;
@@ -621,7 +656,7 @@ value_terminate (void)
 }
 
 void
-value_array_set (Value *array, guint col, guint row, Value *v)
+value_array_set (Value *array, int col, int row, Value *v)
 {
 	g_return_if_fail (v);
 	g_return_if_fail (array->type == VALUE_ARRAY);
@@ -636,7 +671,7 @@ value_array_set (Value *array, guint col, guint row, Value *v)
 }
 
 void
-value_array_resize (Value *v, guint width, guint height)
+value_array_resize (Value *v, int width, int height)
 {
 	int x, y, xcpy, ycpy;
 	Value *newval;
@@ -705,18 +740,95 @@ compare_float_float (Value const *va, Value const *vb)
 		return IS_GREATER;
 }
 
+/**
+ * value_diff :
+ *
+ * @a : value a
+ * @b : value b
+ *
+ * Returns a positive difference between 2 values
+ */
+double
+value_diff (Value const *a, Value const *b)
+{
+	ValueType ta, tb;
+
+	/* Handle trivial and double NULL case */
+	if (a == b)
+		return 0.;
+
+	ta = VALUE_IS_EMPTY (a) ? VALUE_EMPTY : a->type;
+	tb = VALUE_IS_EMPTY (b) ? VALUE_EMPTY : b->type;
+
+	/* string > empty */
+	if (ta == VALUE_STRING) {
+		switch (tb) {
+		/* Strings are > (empty, or number) */
+		case VALUE_EMPTY :
+			if (*a->v_str.val->str == '\0')
+				return 0.;
+			return DBL_MAX;
+
+		/* If both are strings compare as string */
+		case VALUE_STRING :
+		{
+			gint t = strcoll (a->v_str.val->str, b->v_str.val->str);
+			if (t == 0)
+				return 0.;
+		}
+		case VALUE_INTEGER : case VALUE_FLOAT : case VALUE_BOOLEAN :
+		default :
+			return DBL_MAX;
+		}
+
+	} else if (tb == VALUE_STRING) {
+		switch (ta) {
+		/* (empty, or number) < String */
+		case VALUE_EMPTY :
+			if (*b->v_str.val->str == '\0')
+				return 0.;
+
+		case VALUE_INTEGER : case VALUE_FLOAT : case VALUE_BOOLEAN :
+		default :
+			return DBL_MAX;
+		}
+	}
+
+	/* Booleans > all numbers (Why did excel do this ??) */
+	if (ta == VALUE_BOOLEAN && (tb == VALUE_INTEGER || tb == VALUE_FLOAT))
+		return DBL_MAX;
+	if (tb == VALUE_BOOLEAN && (ta == VALUE_INTEGER || ta == VALUE_FLOAT))
+		return DBL_MAX;
+
+	switch ((ta > tb) ? ta : tb) {
+	case VALUE_EMPTY:	/* Empty Empty compare */
+		return 0.;
+
+	case VALUE_BOOLEAN:
+		return (compare_bool_bool (a, b) == IS_EQUAL) ? 0. : DBL_MAX;
+
+	case VALUE_INTEGER: {
+		int const ia = value_get_as_int (a);
+		int const ib = value_get_as_int (b);
+		return abs (ia-ib);
+	}
+
+	case VALUE_FLOAT: {
+		double const da = value_get_as_float (a);
+		double const db = value_get_as_float (b);
+		return fabs (da-db);
+	}
+	default:
+		return TYPE_MISMATCH;
+	}
+}
+
 /**
  * value_compare :
  *
  * @a : value a
  * @b : value b
  * @case_sensitive : are string comparisons case sensitive.
- *
- * Compares two (Value *) and returns one of ValueCompare
- *
- * if pos is non null it will perform implict intersection for
- * cellranges. if case_sensitive is true, be case sensitive on string
- * comparisons.
  */
 ValueCompare
 value_compare (Value const *a, Value const *b, gboolean case_sensitive)
