@@ -39,8 +39,9 @@
 #include "position.h"
 #include "cell.h"
 #include "parse-util.h"
-#include "gnumeric-type-util.h"
+#include "io-context.h"
 
+#include <gal/util/e-util.h>
 #include <gal/widgets/gtk-combo-stack.h>
 #include <locale.h>
 
@@ -65,8 +66,6 @@ Workbook *
 wb_view_workbook (WorkbookView *wbv)
 {
 	g_return_val_if_fail (IS_WORKBOOK_VIEW (wbv), NULL);
-	g_return_val_if_fail (IS_WORKBOOK (wbv->wb), NULL);
-
 	return wbv->wb;
 }
 
@@ -116,18 +115,14 @@ wb_view_sheet_add (WorkbookView *wbv, Sheet *new_sheet)
 }
 
 void
-wb_view_set_attributev (WorkbookView *wbv, GList *list)
+wb_view_set_attribute_list (WorkbookView *wbv, GList *list)
 {
-	gint length, i;
+	GList *l;
 
 	g_return_if_fail (IS_WORKBOOK_VIEW (wbv));
 
-	length = g_list_length(list);
-
-	for (i = 0; i < length; i++){
-		GtkArg *arg = g_list_nth_data (list, i);
-
-		gtk_object_arg_set (GTK_OBJECT (wbv), arg, NULL);
+	for (l = list; l != NULL; l = l->next) {
+		gtk_object_arg_set (GTK_OBJECT (wbv), l->data, NULL);
 	}
 }
 
@@ -373,7 +368,7 @@ wb_view_auto_expr_recalc (WorkbookView *wbv, gboolean display)
 	if (wbv->auto_expr_value_as_string)
 		g_free (wbv->auto_expr_value_as_string);
 	if (v) {
-		const char *val_str = value_peek_string (v);
+		char const *val_str = value_peek_string (v);
 		wbv->auto_expr_value_as_string =
 			g_strconcat (wbv->auto_expr_desc, "=", val_str, NULL);
 		value_release (v);
@@ -527,7 +522,7 @@ workbook_view_init (WorkbookView *wbv, Workbook *opt_wb)
 }
 
 static void
-workbook_view_ctor_class (GtkObjectClass *object_class)
+workbook_view_class_init (GtkObjectClass *object_class)
 {
 	WorkbookViewClass *wbc_class = WORKBOOK_VIEW_CLASS (object_class);
 
@@ -561,11 +556,8 @@ workbook_view_ctor_class (GtkObjectClass *object_class)
 			GTK_TYPE_POINTER);
 }
 
-GNUMERIC_MAKE_TYPE(workbook_view,
-		   "WorkbookView",
-		   WorkbookView,
-		   workbook_view_ctor_class, NULL,
-		   gtk_object_get_type ());
+E_MAKE_TYPE (workbook_view, "WorkbookView", WorkbookView,
+	     workbook_view_class_init, NULL, GTK_TYPE_OBJECT);
 
 WorkbookView *
 workbook_view_new (Workbook *optional_wb)
@@ -576,3 +568,211 @@ workbook_view_new (Workbook *optional_wb)
 	workbook_view_init (view, optional_wb);
 	return view;
 }
+
+/**
+ * wb_view_save_as:
+ * @wbv         : Workbook View
+ * @wbc         : Workbook Control
+ * @fs          : GnumFileSaver object
+ * @file_name   : File name
+ *
+ * Saves @wbv and workbook it's attached to into @file_name file using 
+ * @fs file saver.
+ *
+ * Return value: TRUE if file was successfully saved and FALSE otherwise.
+ */
+gboolean
+wb_view_save_as (WorkbookView *wbv, WorkbookControl *wbc,
+                 GnumFileSaver *fs, gchar const *file_name)
+{
+	gboolean success = FALSE;
+	IOContext *io_context;
+	Workbook *wb;
+
+	g_return_val_if_fail (IS_WORKBOOK_CONTROL (wbc), FALSE);
+	g_return_val_if_fail (file_name != NULL, FALSE);
+	g_return_val_if_fail (IS_GNUM_FILE_SAVER (fs), FALSE);
+
+	wb = wb_control_workbook (wbc);
+	io_context = gnumeric_io_context_new (wbc);
+	gnum_file_saver_save (fs, io_context, wbv, file_name);
+	if (!gnumeric_io_error_occurred (io_context)) {
+		workbook_set_saveinfo (wb, file_name, gnum_file_saver_get_format_level (fs), fs);
+		workbook_set_dirty (wb, FALSE);
+		success = TRUE;
+	} else {
+		gnumeric_io_error_display (io_context);
+		success = FALSE;
+	}
+	gtk_object_destroy (GTK_OBJECT (io_context));
+
+	return success;
+}
+
+/**
+ * wb_view_save:
+ * @wbv         : Workbook View
+ * @wbc         : Workbook Control
+ *
+ * Saves @wbv and workbook it's attached to into file assigned to the
+ * workbook using workbook's file saver. If the workbook has no file
+ * saver assigned to it, default file saver is used instead.
+ *
+ * Return value: TRUE if file was successfully saved and FALSE otherwise.
+ */
+gboolean
+wb_view_save (WorkbookView *wbv, WorkbookControl *wbc)
+{
+	gboolean success = FALSE;
+	IOContext *io_context;
+	Workbook *wb;
+	GnumFileSaver *fs;
+
+	g_return_val_if_fail (IS_WORKBOOK_CONTROL (wbc), FALSE);
+
+	wb = wb_control_workbook (wbc);
+	io_context = gnumeric_io_context_new (wbc);
+	fs = workbook_get_file_saver (wb);
+	if (fs == NULL) {
+		fs = get_default_file_saver ();
+	}
+	if (fs != NULL) {
+		gnum_file_saver_save (fs, io_context, wbv, wb->filename);
+	} else {
+		gnumeric_io_error_save (io_context,
+		_("Default file saver is not available."));
+	}
+	if (!gnumeric_io_error_occurred (io_context)) {
+		workbook_set_dirty (wb, FALSE);
+		success = TRUE;
+	} else {
+		gnumeric_io_error_display (io_context);
+		success = FALSE;
+	}
+	gtk_object_destroy (GTK_OBJECT (io_context));
+
+	return success;
+}
+
+/**
+ * wb_view_open:
+ * @wbv         : Workbook View
+ * @wbc         : Workbook Control
+ * @file_name   : File name
+ * @display_err : should errors messages be generated
+ *
+ * Reads @file_name file, automatically detecting file type and using
+ * appropriate file opener.
+ *
+ * Return value: TRUE if file was successfully read and FALSE otherwise.
+ */
+gboolean
+wb_view_open (WorkbookView *wbv, WorkbookControl *wbc,
+              gchar const *file_name, gboolean display_errors)
+{
+	return wb_view_open_custom (wbv, wbc, NULL, file_name, display_errors);
+}
+
+/**
+ * wb_view_open_custom:
+ * @wbv         : Workbook View
+ * @wbc         : Workbook Control
+ * @fo          : GnumFileOpener object
+ * @file_name   : File name
+ *
+ * Reads @file_name file using given file opener (@fo).
+ *
+ * Return value: TRUE if file was successfully read and FALSE otherwise.
+ */
+gboolean
+wb_view_open_custom (WorkbookView *wbv, WorkbookControl *wbc,
+                     GnumFileOpener const *fo, gchar const *file_name,
+		     gboolean display_errors)
+{
+	Workbook *new_wb = NULL;
+	WorkbookView *new_wbv = NULL;
+
+	g_return_val_if_fail (IS_WORKBOOK_CONTROL (wbc), FALSE);
+	g_return_val_if_fail (fo == NULL || IS_GNUM_FILE_OPENER (fo), FALSE);
+	g_return_val_if_fail (file_name != NULL, FALSE);
+
+	if (g_file_exists (file_name)) {
+		IOContext *io_context = gnumeric_io_context_new (wbc);
+
+		/* Search for an applicable opener */
+		if (fo == NULL) {
+			FileProbeLevel pl;
+			GList *l;
+
+			for (pl = FILE_PROBE_FILE_NAME; pl < FILE_PROBE_LAST && fo == NULL; pl++) {
+				for (l = get_file_openers (); l != NULL; l = l->next) {
+					GnumFileOpener const *tmp_fo = GNUM_FILE_OPENER (l->data);
+					if (gnum_file_opener_probe (tmp_fo, file_name, pl)) {
+						fo = tmp_fo;
+						break;
+					}
+				}
+			}
+		}
+
+		if (fo != NULL) {
+			gboolean old;
+
+			new_wbv = workbook_view_new (NULL);
+			new_wb = wb_view_workbook (new_wbv);
+
+			/* disable recursive dirtying while loading */
+			old = workbook_enable_recursive_dirty (new_wb, FALSE);
+			gnum_file_opener_open (fo, io_context, new_wbv, file_name);
+			workbook_enable_recursive_dirty (new_wb, old);
+
+			if (!gnumeric_io_error_occurred (io_context) &&
+			    workbook_sheet_count (new_wb) == 0)
+				gnumeric_io_error_read (io_context, _("No sheets in workbook."));
+
+			if (!gnumeric_io_error_occurred (io_context)) {
+				workbook_set_dirty (new_wb, FALSE);
+			} else {
+				gtk_object_destroy (GTK_OBJECT (new_wb));
+				new_wb = NULL;
+				new_wbv = NULL;
+			}
+		} else
+			gnumeric_io_error_read (io_context, _("Unsupported file format."));
+
+		if (gnumeric_io_error_occurred (io_context)) {
+			gnumeric_io_error_display (io_context);
+		}
+
+		gtk_object_destroy (GTK_OBJECT (io_context));
+	} else {
+		new_wb = workbook_new_with_sheets (1);
+		new_wbv = workbook_view_new (new_wb);
+		workbook_set_saveinfo (new_wb, file_name, FILE_FL_NEW, NULL);
+	}
+
+	if (new_wbv != NULL) {
+		Workbook *old_wb;
+
+		g_return_val_if_fail (new_wb != NULL, FALSE);
+
+		old_wb = wb_control_workbook (wbc);
+		if (workbook_is_pristine (old_wb)) {
+			gtk_object_ref (GTK_OBJECT (wbc));
+			workbook_unref (old_wb);
+			workbook_control_set_view (wbc, new_wbv, NULL);
+			workbook_control_init_state (wbc);
+		} else
+			(void) wb_control_wrapper_new (wbc, new_wbv, NULL);
+
+		workbook_recalc (new_wb);
+
+		g_return_val_if_fail (!workbook_is_dirty (new_wb), FALSE);
+
+		sheet_update (wb_view_cur_sheet (new_wbv));
+		return TRUE;
+	} else {
+		g_return_val_if_fail (new_wb == NULL, FALSE);
+		return FALSE;
+	}
+}
